@@ -71,6 +71,23 @@ def convert(args):
     return 0
 
 
+def dump_xlsx_template(args):
+    """creates a blank XSLX spreadsheet based on provided schema"""
+    schema = args.schema
+    output = args.output
+    engine = args.engine
+    schema_dir, schema_fname = os.path.split(schema)
+    schema_fname, _ = os.path.splitext(schema_fname)
+    data = Data()
+    data.read_schema(args.schema)
+    breakpoint()
+    cols = data.dump_template()
+    with pd.ExcelWriter(output) as writer:
+        for tabname, df in cols.items():
+            df.to_excel(writer, sheet_name=tabname, index=False)
+    return output
+
+
 def parse_args(args):
     """Console script"""
     # https://docs.python.org/3/library/argparse.html
@@ -80,12 +97,11 @@ def parse_args(args):
         "-v", "--verbose", action="store_true", help="Show Version and exit"
     )
     parser.set_defaults(process=version)
-    # ========================================================================
-    # subparsers
-    # ========================================================================
+
     subparsers = parser.add_subparsers(help="sub-command help")
-    # ------------------------------------------------------------------------
-    # create the parser for the "a" command
+    # ========================================================================
+    # parser_converter
+    # ========================================================================
     parser_convert = subparsers.add_parser("convert", help="convert")
     parser_convert.add_argument("input", type=str, help="path to input file")
     parser_convert.add_argument("-o", "--output", type=str, help="path to output file")
@@ -99,8 +115,10 @@ def parse_args(args):
         help="pandas XLSX reader engine",
     )
     parser_convert.set_defaults(process=convert)
-    # ------------------------------------------------------------------------
+    # =========================================================================
+    # parser_format:
     # inplace formatting
+    # =========================================================================
     parser_format = subparsers.add_parser("format", help="format file inplace")
     parser_format.add_argument("input", type=str, help="path to input file")
     parser_format.add_argument(
@@ -112,6 +130,30 @@ def parse_args(args):
         help="pandas XLSX reader engine",
     )
     parser_format.set_defaults(process=format)
+    # =========================================================================
+    # parser_dump
+    # dump parser
+    # =========================================================================
+    parser_dumper = subparsers.add_parser(
+        "dump-template", help="dump template based on provided schema"
+    )
+    parser_dumper.add_argument("schema", type=str, help="path to schema file")
+    parser_dumper.add_argument(
+        "-o",
+        "--output",
+        type=str,
+        default="template.xlsx",
+        help="output filepath",
+    )
+    parser_dumper.add_argument(
+        "-e",
+        "--engine",
+        type=str,
+        default="xlrd",
+        choices=("xlrd", "openpyxl"),
+        help="pandas XLSX reader engine",
+    )
+    parser_dumper.set_defaults(process=dump_xlsx_template)
 
     # ========================================================================
     # return default func
