@@ -50,6 +50,66 @@ PD_SENSIBLE_TESTING = {
 }
 
 
+# =============================================================================
+# functional read() methods
+# =============================================================================
+def _cfg_section_to_df(tabname, data, len_prefix):
+    """eg.
+    tabname: "names"
+    data: {'_row_0': {'id': 1, 'Name': 'Doe', 'Firstname': 'John'}, ...}
+    """
+    # transform "{ROW_PREFIX}n" -> index
+    try:
+        index = [int(ix[len_prefix:]) for ix in data.keys()]
+    except:
+        __import__("pdb").set_trace()
+    df = pd.DataFrame(data.values(), index=index)
+    df = df.replace("", np.nan, regex=False)
+    df.columns = [c.replace(":::", "\n") for c in df.columns]
+    return df
+
+
+def read_data(fpath, sheet_name=None, engine=DEFAULT_EXCEL_ENGINE):
+    """mimic pandas.read_excel() function"""
+    fmts = {
+        ".xlsx": (read_excel, {"engine": engine}),
+        ".cfg": (read_configobj, {}),
+        ".ini": (read_configobj, {}),
+    }
+    frootname, ext = os.path.splitext(fpath)
+    func, kwargs = fmts[ext]
+    return func(fpath=fpath, sheet_name=sheet_name, **kwargs)
+
+
+def read_excel(fpath, sheet_name=None, engine=DEFAULT_EXCEL_ENGINE):
+    """read excel file using pandas"""
+    return pd.read_excel(fpath, sheet_name=sheet_name, engine=engine)
+
+
+def read_configobj(fpath, sheet_name=None, row_prefix="sniff"):
+    ret = {}
+    config = ConfigObj(fpath, indent_type="    ", unrepr=True, write_empty_values=True)
+    # transform sections to tabs and data to columns...
+    if row_prefix == "sniff":
+        row_prefix = _sniff_row_prefix(config[config.sections[0]].sections[0])
+    else:
+        row_prefix = ROW_PREFIX
+    for tabname, data in config.items():
+        ret[tabname] = _cfg_section_to_df(tabname, data, len(row_prefix))
+    # filter out sheet_name
+    if sheet_name is None:
+        return ret
+    elif isinstance(sheet_name, str):
+        return ret[sheet_name]
+    else:
+        raise NotImplementedError("not imp")
+
+
+# =============================================================================
+# Object wrapper
+# =============================================================================
+
+
 class IOMixin:
     """Input/ouput for grid data"""
 
@@ -129,11 +189,11 @@ class IOMixin:
     # EXCEL io
     # ========================================================================
 
-    def read_excel(self, fpath, engine=DEFAULT_EXCEL_ENGINE):
+    def read_excel(self, fpath, sheet_name=None, engine=DEFAULT_EXCEL_ENGINE):
         """read excel-like file and store all the tabs
         as pandas DataFrame values"""
         self.reset()
-        self._data = pd.read_excel(fpath, sheet_name=None, engine=engine)
+        self._data = read_excel(fpath, sheet_name=sheet_name, engine=engine)
 
     def to_excel(self, fpath):
         with pd.ExcelWriter(fpath) as writer:
@@ -231,42 +291,14 @@ class IOMixin:
         config.write()
         return fpath
 
-    def read_configobj(self, fpath, row_prefix="sniff"):
+    def read_configobj(self, fpath, sheet_name=None, row_prefix="sniff"):
         if not IS_CONFIGOBJ:
             raise ValueError(
                 "Cannot export to configobj. Please install configobj before"
             )
         self.reset()
-        config = ConfigObj(
-            fpath, indent_type="    ", unrepr=True, write_empty_values=True
-        )
-        # transform sections to tabs and data to columns...
-        if row_prefix == "sniff":
-            row_prefix = _sniff_row_prefix(config[config.sections[0]].sections[0])
-            # # sniff row_prefix
-            # first_item = config[config.sections[0]].sections[0]  # eg.: "_row_0"
-            # row_prefix = re.match(r"^(.*)\d+$", first_item).group(1)
-        else:
-            row_prefix = ROW_PREFIX
-        for tabname, data in config.items():
-            self._data[tabname] = _cfg_section_to_df(tabname, data, len(row_prefix))
+        self._data = read_configobj(fpath, sheet_name=sheet_name)
 
 
 def _sniff_row_prefix(sec_title):
     return re.match(r"^(.*)\d+$", sec_title).group(1)
-
-
-def _cfg_section_to_df(tabname, data, len_prefix):
-    """eg.
-    tabname: "names"
-    data: {'_row_0': {'id': 1, 'Name': 'Doe', 'Firstname': 'John'}, ...}
-    """
-    # transform "{ROW_PREFIX}n" -> index
-    try:
-        index = [int(ix[len_prefix:]) for ix in data.keys()]
-    except:
-        __import__("pdb").set_trace()
-    df = pd.DataFrame(data.values(), index=index)
-    df = df.replace("", np.nan, regex=False)
-    df.columns = [c.replace(':::', '\n') for c in df.columns]
-    return df
