@@ -45,84 +45,120 @@ life_nb:
     schema = v.build()
     expected_schema = {
         "$schema": "http://json-schema.org/draft-07/schema#",
-        "type": "object",
         "properties": {
-            "id": {"type": "array", "items": {"type": "integer"}, "uniqueItems": True},
-            "name": {
-                "type": "array",
-                "items": {"type": "string"},
-                "uniqueItems": False,
-            },
-            "firstname": {
-                "type": "array",
-                "items": {"anyOf": [{"type": "string"}, {"type": "null"}]},
-                "uniqueItems": False,
-            },
             "age": {
-                "type": "array",
                 "items": {
                     "anyOf": [{"type": "integer"}, {"type": "null"}],
                     "minimum": 0,
                 },
+                "type": "array",
                 "uniqueItems": False,
             },
+            "firstname": {
+                "items": {"anyOf": [{"type": "string"}, {"type": "null"}]},
+                "type": "array",
+                "uniqueItems": False,
+            },
+            "id": {"items": {"type": "integer"}, "type": "array", "uniqueItems": True},
             "life_nb": {
+                "items": {"maximum": 4, "type": "integer"},
+                "type": "array",
+                "uniqueItems": False,
+            },
+            "name": {
+                "items": {"type": "string"},
                 "type": "array",
-                "items": {"type": "integer", "maximum": 4},
                 "uniqueItems": False,
             },
         },
+        "required": ["id", "name", "life_nb"],
+        "type": "object",
     }
+
     assert schema == expected_schema
 
 
 def test_DFS_01_validation():
     """test DataFrameSchematizer class validation.
     DataFrameSchematizer class is purely internal and wrapped by VData or IVData
+
+
+    This test-case mainly check **default** values on a simple one-row dataframe
+
+    | Value        | default | mandatory | result    | check |
+    |--------------+---------+-----------+-----------+-------|
+    | blank or nan |         | True      | FAIL      | da    |
+    | blank or nan |         | False     | blank nan | db    |
+    | blank or nan | 1       | True      | 1         | dc    |
+    | blank or nan | 1       | False     | 1         | dd    |
+    | 2            |         | True      | 2         | de    |
+    | 2            |         | False     | 2         | df    |
+    | 2            | 1       | True      | 2         | dg    |
+    | 2            | 1       | False     | 2         | dh    |
     """
     v = DataFrameSchematizer()
     columns_specs = """\
 ---
-id:
-  types: integer
-  unique: true
-  mandatory: true
-name:
-  types: string
-  mandatory: true
-firstname:
-  types: string
-age:
-  types: integer
-  minimum: 0
-life_nb:
+da:
   types: integer
   mandatory: true
-  maximum: 4
+db:
+  types: integer
+  mandatory: false
+dc:
+  types: integer
+  mandatory: true
+  default: 1
+dd:
+  types: integer
+  mandatory: false
+  default: 1
+de:
+  types: integer
+  mandatory: true
+df:
+  types: integer
+  mandatory: false
+dg:
+  types: integer
+  mandatory: true
+  default: 1
+dh:
+  types: integer
+  mandatory: false
+  default: 1
 """
     v.add_columns(columns_specs)
-    df = df = pd.DataFrame(
+    df = pd.DataFrame(
         {
-            "id": {7: 0, 1: 1, 2: 5},
-            "name": {7: "Doe", 1: "Fante", 2: "Mercury"},
-            "firstname": {7: "John", 2: "Freddy", 1: "Richard"},
-            "age": {7: "42", 1: 22},
-            "life_nb": {7: 5, 1: "hg", 2: 15},
+            "da": {1: None},
+            "db": {},
+            "dc": {1: np.nan},
+            "dd": {1: None},
+            "de": {1: 2},
+            "df": {1: 2},
+            "dg": {1: 2},
+            "dh": {1: 2},
         }
     )
-    is_valid, errors = v.validate(df)
+    df, is_valid, errors = v.validate(df)
     assert is_valid is False
-    expected_errors = {
-        ("age", 0): [
-            "'42' is not valid under any of the given schemas",
-            "'42' is not of type 'integer'",
-            "'42' is not of type 'null'",
-        ],
-        ("life_nb", 0): ["5 is greater than the maximum of 4"],
-        ("life_nb", 1): ["'hg' is not of type 'integer'"],
-        ("life_nb", 2): ["15 is greater than the maximum of 4"],
-    }
+    expected_errors = {("da", 0): ["None is not of type 'integer'"]}
     assert errors == expected_errors
+    # check that values have been correctly filled
+    expected_df = pd.DataFrame(
+        {
+            "da": {1: None},
+            "db": {1: np.nan},
+            "dc": {1: 1},
+            "dd": {1: 1},
+            "de": {1: 2},
+            "df": {1: 2},
+            "dg": {1: 2},
+            "dh": {1: 2},
+        }
+    )
+    assert pd.testing.assert_frame_equal(df, expected_df, check_dtype=False) is None
 
 
 def test_VData_00():
@@ -174,7 +210,7 @@ def test_VData_00():
         ("life_nb", 1): ["'hg' is not of type 'integer'"],
         ("life_nb", 2): ["15 is greater than the maximum of 4"],
     }
-    is_valid, errors = data.validate_tab("test")
+    df, is_valid, errors = data.validate_tab("test")
     assert is_valid is False
     assert errors == expected_report
 
@@ -206,12 +242,12 @@ def test_IVData_00(datadir):
     data.read_schema(os.path.join(indir, "test_00.schema.yaml"))
     for tab in ("names", "cars", "empty"):
         print('checking "%s"' % tab, end="... ")
-        is_ok, errors = data.validate_tab(tab)
+        df, is_ok, errors = data.validate_tab(tab)
         try:
             assert is_ok is True
         except:
             print("OUPS!")
-            __import__("pdb").set_trace()
+            breakpoint()
         else:
             print("OK")
         assert errors == {}
@@ -230,12 +266,11 @@ def test_IVData_00(datadir):
         data_new.read_schema(os.path.join(indir, "test_00.schema.yaml"))
         for tab in data._data.keys():
             print('checking "%s"' % tab, end="... ")
-            is_ok, errors = data_new.validate_tab(tab)
+            df, is_ok, errors = data_new.validate_tab(tab)
             try:
                 assert is_ok is True
             except:
                 print("OUPS!")
-                __import__("pdb").set_trace()
             else:
                 print("OK")
             assert errors == {}
@@ -256,32 +291,28 @@ def test_IVData_02(datadir):
     data.read_excel(os.path.join(indir, "test_00.xlsx"))
     data.read_schema(os.path.join(indir, "test_00.schema2.yaml"))
     ret = data.validate()
-    assert ret == {"cars": {("Year", 2): ["None is not of type 'integer'"]}}
+    assert ret == {
+        "cars": {("Year", 2): ["None is not of type 'integer'"]},
+        "names": {"general": ["'name' is a required property"]},
+    }
 
 
 def test_IVData_03(datadir):
-    """ make schema a bit more complex """
-    indir, outdir = datadir
-    data = IVData()
-    data.read_excel(os.path.join(indir, "test_01.xlsx"))
-    data.read_schema(os.path.join(indir, "test_01.schema2.yaml"))
-    ret = data.validate()
-    assert len(ret) == 0
-
-
-def test_IVData_04(datadir):
     """ make schema a bit more complex: add generic keys """
     indir, outdir = datadir
     data = IVData()
     data.read_excel(os.path.join(indir, "test_02.xlsx"))
     data.read_schema(os.path.join(indir, "test_02.schema.yaml"), debug=True)
     ret = data.validate()
-    assert len(ret) == 1
+    print(ret)  # TODO: remove me!
+    assert len(ret) == 2
     expected_errors = {
+        "names": {"general": ["'life_nb' is a required property"]},
         "french_cars": {
             ("brand", 2): [
                 "'Citroën' is not one of ['Peugeot', 'Toyota', 'Ford', 'Renault']"
             ]
-        }
+        },
     }
+
     assert ret == expected_errors
